==> cappuccino-2.0.x-dev.tar.gz: cappuccino/web/profiles/cappuccino/modules/ino_pt_list/modules/ino_pt_list_term_reference_filter/src/Plugin/Field/FieldWidget/TermReferenceFilterWidget.php <==
<?php

namespace Drupal\ino_pt_list_term_reference_filter\Plugin\Field\FieldWidget;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class TermReferenceFilterWidget.
 *
 * @FieldWidget(
 *   id = "ino_list_term_reference_filter",
 *   label = @Translation("Term reference filter widget"),
 *   field_types = {
 *     "ino_list_term_reference_filter"
 *   }
 * )
 */
class TermReferenceFilterWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * Constructs a TermReferenceFilterWidget object.
   *
   * @param string $plugin_id
   *   The plugin_id for the widget.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the widget is associated.
   * @param array $settings
   *   The widget settings.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager service.
   */
  public function __construct(string $plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, EntityTypeManagerInterface $entity_type_manager, EntityFieldManagerInterface $entity_field_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager')
    );
  }

  /**
   * Gets a paragraph form's parent entity.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   *
   * @return null|object
   *   Node or taxonomy term.
   */
  private function getParentEntity(FormStateInterface $form_state) {
    $entity = NULL;
    $build_info = $form_state->getBuildInfo();
    if (!empty($build_info) && isset($build_info['callback_object'])) {
      /** @var \Drupal\Core\Entity\Entity $entity */
      $entity = $build_info['callback_object']->getEntity();
      if (NULL !== $entity && is_a($entity, 'Drupal\paragraphs\Entity\Paragraph')) {
        // This is a Paragraphs Edit form.
        $entity = $entity->getParentEntity();
      }
    }

    return $entity;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *   Thrown if the entity type doesn't exist.
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   *   Thrown if the storage handler couldn't be loaded.
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $vocabularies = $this->entityTypeManager
      ->getStorage('taxonomy_vocabulary')
      ->loadByProperties();
    if (empty($vocabularies)) {
      return $element;
    }
    $element['settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Taxonomy terms filtering'),
      '#open' => FALSE,
    ];
    // Get vocabularies referenced by the parent entity (if any).
    $parent_bundle_label = '';
    $vids_in_parent = [];
    $parent_entity = $this->getParentEntity($form_state);
    if (in_array($parent_entity->getEntityTypeId(), ['node', 'taxonomy_term'])) {
      $bundle_id = $parent_entity->getEntityType()->getBundleEntityType();
      if ($bundle_id != NULL) {
        $parent_bundle_label = $this->entityTypeManager
          ->getStorage($bundle_id)
          ->load($parent_entity->bundle())
          ->label();
      }
      $field_info = $parent_entity->getFieldDefinitions();
      foreach ($field_info as $field_item) {
        /** @var \Drupal\field\Entity\FieldConfig $field_item */
        if ('entity_reference' == $field_item->getType()) {
          $settings = $field_item->getSettings();
          if (isset($settings['target_type'], $settings['handler_settings']['target_bundles'])
            && 'taxonomy_term' == $settings['target_type']) {
            $vids_in_parent += $settings['handler_settings']['target_bundles'];
          }
        }
      }
    }
    // Bundles and term filters + states API visibility conditions.
    $states_path_entity_type = ino_content_editing_form_element_states_path($form, 'field_ph_list_entity_type');
    // TODO: replace hardcoded field names.
    $node_type_field = $this->getFieldSettings()['trfw_node_type_field'];
    if (isset($form[$node_type_field])) {
      $states_path_node_types = ino_content_editing_form_element_states_path($form, $node_type_field);
      $node_type_ids = array_keys($form[$node_type_field]['widget']['#options']);
      $vids_unrelated_node_types = $this->getVocabularyUnrelatedBundles($vocabularies, $node_type_ids, 'node');
      // If a vocabulary is used by all the bundles never hide all term filters.
      $diff = array_diff_key($vocabularies, $vids_unrelated_node_types);
      if (empty($diff)) {
        // Set states to hide all term filters for some bundles.
        $bundles_without_term_ref = count($vids_unrelated_node_types) > 1
          ? call_user_func_array('array_intersect', $vids_unrelated_node_types)
          : current($vids_unrelated_node_types);
        foreach ($bundles_without_term_ref as $bundle) {
          $element['settings']['#states']['invisible'][] = [
            ":input[name='{$states_path_entity_type}']" => ['value' => 'node'],
            ":input[name='{$states_path_node_types}[{$bundle}]']" => ['checked' => TRUE],
          ];
        }
      }
    }
    $taxonomy_type_field = $this->getFieldSettings()['trfw_taxonomy_type_field'];
    if (isset($form[$taxonomy_type_field])) {
      $states_path_vids = ino_content_editing_form_element_states_path($form, $taxonomy_type_field);
      $vocabulary_ids = array_keys($form[$taxonomy_type_field]['widget']['#options']);
      $vids_unrelated_vids = $this->getVocabularyUnrelatedBundles($vocabularies, $vocabulary_ids, 'taxonomy_term');
      // If a vocabulary is used by all the bundles never hide all term filters.
      $diff = array_diff_key($vocabularies, $vids_unrelated_vids);
      if (empty($diff)) {
        // Set states to hide all term filters for some bundles.
        $bundles_without_term_ref = count($vids_unrelated_vids) > 1
          ? call_user_func_array('array_intersect', $vids_unrelated_vids)
          : current($vids_unrelated_vids);
        foreach ($bundles_without_term_ref as $bundle) {
          $element['settings']['#states']['invisible'][] = [
            ":input[name='{$states_path_entity_type}']" => ['value' => 'term'],
            ":input[name='{$states_path_vids}[{$bundle}]']" => ['checked' => TRUE],
          ];
        }
      }
    }
    // Get saved value to set the default values.
    $saved_value = $items->get($delta)->getValue();
    // Full field names within the form for states API.
    if (empty($element['#field_parents'])) {
      // The field has no parents in Paragraph Edit forms.
      // TODO: replace hardcoded field names.
      $input_name_start = 'field_ph_list_term_filters';
    }
    else {
      $field_path = array_merge($element['#field_parents'], ['field_ph_list_term_filters']);
      $input_name_start = array_shift($field_path) . '[' . implode('][', $field_path) . ']';
    }
    // Add the vocabulary filters with visibility form states.
    foreach ($vocabularies as $vid => $vocabulary) {
      $default_use_global = !empty($saved_value['settings'][$vid]['use_global']) && $saved_value['settings'][$vid]['use_global'] === 1;
      $default_referenced_terms = !empty($saved_value['settings'][$vid]['term_reference'])
        ? $saved_value['settings'][$vid]['term_reference']
        : [];
      $tids = [];
      foreach ($default_referenced_terms as $term) {
        $tids[] = $term['target_id'];
      }
      $default_referenced_terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadMultiple($tids);
      // Build states filters.
      $states = [];
      if (isset($states_path_node_types, $vids_unrelated_node_types[$vid])) {
        // Vocabulary isn't referenced by all the node types.
        // Form states to hide filter for unrelated node type.
        foreach ($vids_unrelated_node_types[$vid] as $node_type) {
          $states[] = [
            ":input[name='{$states_path_entity_type}']" => ['value' => 'node'],
            ":input[name='{$states_path_node_types}[{$node_type}]']" => ['checked' => TRUE],
          ];
          $states[] = 'or';
        }
      }
      if (isset($states_path_vids, $vids_unrelated_vids[$vid])) {
        // Vocabulary isn't referenced by all the vocabularies.
        // Form states to hide filter for unrelated vocabulary.
        foreach ($vids_unrelated_vids[$vid] as $unrelated_vid) {
          $states[] = [
            ":input[name='{$states_path_entity_type}']" => ['value' => 'term'],
            ":input[name='{$states_path_vids}[{$unrelated_vid}]']" => ['checked' => TRUE],
          ];
          $states[] = 'or';
        }
      }
      $element['settings'][$vid] = [
        '#title' => $vocabulary->label(),
        '#type' => 'fieldset',
        '#states' => [
          'invisible' => array_slice($states, 0, -1),
        ],
      ];
      // Get checkbox input name full.
      $input_name_full = $input_name_start . '[' . $delta . '][settings][' . $vid . '][use_global]';
      // Add extra filter to form element.
      $states[] = [":input[name='{$input_name_full}']" => ['checked' => TRUE]];
      if (isset($vids_in_parent[$vid])) {
        // This vocabulary is referenced by the parent entity.
        $element['settings'][$vid]['use_global'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Use current taxonomy terms'),
          '#default_value' => $default_use_global,
          '#attributes' => [
            'class' => ["use_global_{$delta}_{$vid}"],
          ],
        ];
      }
      else {
        // This vocabulary is not referenced by the parent entity.
        $element['settings'][$vid]['not_use_global'] = [
          '#type' => 'markup',
          '#markup' => $this->t('The %bundle entities do not use this vocabulary.', ['%bundle' => $parent_bundle_label]),
        ];
        $element['settings'][$vid]['use_global'] = [
          '#type' => 'value',
          '#value' => 0,
        ];
      }
      $element['settings'][$vid]['term_reference'] = [
        '#type' => 'entity_autocomplete',
        '#description' => $this->t("Separate terms using a comma ','."),
        '#target_type' => 'taxonomy_term',
        '#selection_handler' => 'default:taxonomy_term',
        '#selection_settings' => [
          'target_bundles' => [
            $vid => $vid,
          ],
          'sort' => [
            'field' => 'name',
            'direction' => 'ASC',
          ],
          'auto_create' => FALSE,
          'auto_create_bundle' => NULL,
          'match_operator' => 'CONTAINS',
        ],
        '#default_value' => $default_referenced_terms,
        '#tags' => TRUE,
        '#maxlength' => 512,
        '#states' => [
          'invisible' => $states,
        ],
      ];
      if (!empty($default_referenced_terms) || $default_use_global === TRUE) {
        $element['settings']['#open'] = TRUE;
      }
    }

    return $element;
  }

  /**
   * List of vocabularies and all the bundles that don't reference them.
   *
   * @param array $vocabularies
   *   Vocabulary list.
   * @param array $entity_bundle_ids
   *   List of bundles for this entity type.
   * @param string $entity_type
   *   Id of the entity_type.
   *
   * @return array
   *   Return non-related bundles for each vocabulary.
   */
  protected function getVocabularyUnrelatedBundles(array $vocabularies, array $entity_bundle_ids, $entity_type) {
    $vocab_entity_types = [];
    foreach ($vocabularies as $id => $vocab) {
      $vocab_entity_types[$id] = array_combine($entity_bundle_ids, $entity_bundle_ids);
    }
    foreach ($entity_bundle_ids as $entity_bundle) {
      if ($entity_bundle == '_none') {
        continue;
      }
      $fields = $this->entityFieldManager->getFieldDefinitions($entity_type, $entity_bundle);
      foreach ($fields as $field_info) {
        /** @var \Drupal\field\Entity\FieldConfig $fieldItem */
        if ('entity_reference' == $field_info->getType()) {
          $settings = $field_info->getSettings();
          if (isset($settings['target_type']) && 'taxonomy_term' == $settings['target_type']) {
            if (!empty($settings['handler_settings']['target_bundles'])) {
              foreach ($settings['handler_settings']['target_bundles'] as $vocab_name) {
                unset($vocab_entity_types[$vocab_name][$entity_bundle]);
              }
            }
          }
        }
      }
    }
    $vocab_entity_types = array_filter($vocab_entity_types);

    return $vocab_entity_types;
  }

}

==> cappuccino-3.0.x-dev.tar.gz: cappuccino/web/profiles/cappuccino/modules/ino_pt_list/modules/ino_pt_list_term_reference_filter/src/Plugin/Field/FieldWidget/TermReferenceFilterWidget.php <==
<?php

namespace Drupal\ino_pt_list_term_reference_filter\Plugin\Field\FieldWidget;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class TermReferenceFilterWidget.
 *
 * @FieldWidget(
 *   id = "ino_list_term_reference_filter",
 *   label = @Translation("Term reference filter widget"),
 *   field_types = {
 *     "ino_list_term_reference_filter"
 *   }
 * )
 */
class TermReferenceFilterWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * Constructs a TermReferenceFilterWidget object.
   *
   * @param string $plugin_id
   *   The plugin_id for the widget.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the widget is associated.
   * @param array $settings
   *   The widget settings.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager service.
   */
  public function __construct(string $plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, EntityTypeManagerInterface $entity_type_manager, EntityFieldManagerInterface $entity_field_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager')
    );
  }

  /**
   * Gets a paragraph form's parent entity.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   *
   * @return null|object
   *   Node or taxonomy term.
   */
  private function getParentEntity(FormStateInterface $form_state) {
    $entity = NULL;
    $build_info = $form_state->getBuildInfo();
    if (!empty($build_info) && isset($build_info['callback_object'])) {
      /** @var \Drupal\Core\Entity\Entity $entity */
      $entity = $build_info['callback_object']->getEntity();
      if (NULL !== $entity && is_a($entity, 'Drupal\paragraphs\Entity\Paragraph')) {
        // This is a Paragraphs Edit form.
        $entity = $entity->getParentEntity();
      }
    }

    return $entity;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *   Thrown if the entity type doesn't exist.
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   *   Thrown if the storage handler couldn't be loaded.
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $vocabularies = $this->entityTypeManager
      ->getStorage('taxonomy_vocabulary')
      ->loadByProperties();
    if (empty($vocabularies)) {
      return $element;
    }
    $element['settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Taxonomy terms filtering'),
      '#open' => FALSE,
    ];
    // Get vocabularies referenced by the parent entity (if any).
    $parent_bundle_label = '';
    $vids_in_parent = [];
    $parent_entity = $this->getParentEntity($form_state);
    if (in_array($parent_entity->getEntityTypeId(), ['node', 'taxonomy_term'])) {
      $bundle_id = $parent_entity->getEntityType()->getBundleEntityType();
      if ($bundle_id != NULL) {
        $parent_bundle_label = $this->entityTypeManager
          ->getStorage($bundle_id)
          ->load($parent_entity->bundle())
          ->label();
      }
      $field_info = $parent_entity->getFieldDefinitions();
      foreach ($field_info as $field_item) {
        /** @var \Drupal\field\Entity\FieldConfig $field_item */
        if ('entity_reference' === $field_item->getType()) {
          $settings = $field_item->getSettings();
          if (isset($settings['target_type'], $settings['handler_settings']['target_bundles'])
            && 'taxonomy_term' === $settings['target_type']) {
            $vids_in_parent += $settings['handler_settings']['target_bundles'];
          }
        }
      }
    }
    // Bundles and term filters + states API visibility conditions.
    $states_path_entity_type = ino_content_editing_form_element_states_path($form, 'field_ph_list_entity_type');
    // TODO: replace hardcoded field names.
    $node_type_field = $this->getFieldSettings()['trfw_node_type_field'];
    if (isset($form[$node_type_field])) {
      $states_path_node_types = ino_content_editing_form_element_states_path($form, $node_type_field);
      $node_type_ids = array_keys($form[$node_type_field]['widget']['#options']);
      $vids_unrelated_node_types = $this->getVocabularyUnrelatedBundles($vocabularies, $node_type_ids, 'node');
      // If a vocabulary is used by all the bundles never hide all term filters.
      $diff = array_diff_key($vocabularies, $vids_unrelated_node_types);
      if (empty($diff)) {
        // Set states to hide all term filters for some bundles.
        $bundles = reset($vids_unrelated_node_types);
        $tags = next($vids_unrelated_node_types);
        $bundles_without_term_ref = count($vids_unrelated_node_types) > 1
          ? array_intersect($bundles, $tags)
          : current($vids_unrelated_node_types);
        foreach ($bundles_without_term_ref as $bundle) {
          $element['settings']['#states']['invisible'][] = [
            ":input[name='{$states_path_entity_type}']" => ['value' => 'node'],
            ":input[name='{$states_path_node_types}[{$bundle}]']" => ['checked' => TRUE],
          ];
        }
      }
    }
    $taxonomy_type_field = $this->getFieldSettings()['trfw_taxonomy_type_field'];
    if (isset($form[$taxonomy_type_field])) {
      $states_path_vids = ino_content_editing_form_element_states_path($form, $taxonomy_type_field);
      $vocabulary_ids = array_keys($form[$taxonomy_type_field]['widget']['#options']);
      $vids_unrelated_vids = $this->getVocabularyUnrelatedBundles($vocabularies, $vocabulary_ids, 'taxonomy_term');
      // If a vocabulary is used by all the bundles never hide all term filters.
      $diff = array_diff_key($vocabularies, $vids_unrelated_vids);
      if (empty($diff)) {
        // Set states to hide all term filters for some bundles.
        $bundles = reset($vids_unrelated_vids);
        $tags = next($vids_unrelated_vids);
        $bundles_without_term_ref = count($vids_unrelated_vids) > 1
          ? array_intersect($bundles, $tags)
          : current($vids_unrelated_vids);
        foreach ($bundles_without_term_ref as $bundle) {
          $element['settings']['#states']['invisible'][] = [
            ":input[name='{$states_path_entity_type}']" => ['value' => 'term'],
            ":input[name='{$states_path_vids}[{$bundle}]']" => ['checked' => TRUE],
          ];
        }
      }
    }
    // Get saved value to set the default values.
    $saved_value = $items->get($delta)->getValue();
    // Full field names within the form for states API.
    if (empty($element['#field_parents'])) {
      // The field has no parents in Paragraph Edit forms.
      // TODO: replace hardcoded field names.
      $input_name_start = 'field_ph_list_term_filters';
    }
    else {
      $field_path = array_merge($element['#field_parents'], ['field_ph_list_term_filters']);
      $input_name_start = array_shift($field_path) . '[' . implode('][', $field_path) . ']';
    }
    // Add the vocabulary filters with visibility form states.
    foreach ($vocabularies as $vid => $vocabulary) {
      $default_use_global = !empty($saved_value['settings'][$vid]['use_global']) && $saved_value['settings'][$vid]['use_global'] === 1;
      $default_referenced_terms = !empty($saved_value['settings'][$vid]['term_reference'])
        ? $saved_value['settings'][$vid]['term_reference']
        : [];
      $tids = [];
      foreach ($default_referenced_terms as $term) {
        $tids[] = $term['target_id'];
      }
      $default_referenced_terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadMultiple($tids);
      // Build states filters.
      $states = [];
      if (isset($states_path_node_types, $vids_unrelated_node_types[$vid])) {
        // Vocabulary isn't referenced by all the node types.
        // Form states to hide filter for unrelated node type.
        foreach ($vids_unrelated_node_types[$vid] as $node_type) {
          $states[] = [
            ":input[name='{$states_path_entity_type}']" => ['value' => 'node'],
            ":input[name='{$states_path_node_types}[{$node_type}]']" => ['checked' => TRUE],
          ];
          $states[] = 'or';
        }
      }
      if (isset($states_path_vids, $vids_unrelated_vids[$vid])) {
        // Vocabulary isn't referenced by all the vocabularies.
        // Form states to hide filter for unrelated vocabulary.
        foreach ($vids_unrelated_vids[$vid] as $unrelated_vid) {
          $states[] = [
            ":input[name='{$states_path_entity_type}']" => ['value' => 'term'],
            ":input[name='{$states_path_vids}[{$unrelated_vid}]']" => ['checked' => TRUE],
          ];
          $states[] = 'or';
        }
      }
      $element['settings'][$vid] = [
        '#title' => $vocabulary->label(),
        '#type' => 'fieldset',
        '#states' => [
          'invisible' => array_slice($states, 0, -1),
        ],
      ];
      // Get checkbox input name full.
      $input_name_full = $input_name_start . '[' . $delta . '][settings][' . $vid . '][use_global]';
      // Add extra filter to form element.
      $states[] = [":input[name='{$input_name_full}']" => ['checked' => TRUE]];
      if (isset($vids_in_parent[$vid])) {
        // This vocabulary is referenced by the parent entity.
        $element['settings'][$vid]['use_global'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Use current taxonomy terms'),
          '#default_value' => $default_use_global,
          '#attributes' => [
            'class' => ["use_global_{$delta}_{$vid}"],
          ],
        ];
      }
      else {
        // This vocabulary is not referenced by the parent entity.
        $element['settings'][$vid]['not_use_global'] = [
          '#type' => 'markup',
          '#markup' => $this->t('The %bundle entities do not use this vocabulary.', ['%bundle' => $parent_bundle_label]),
        ];
        $element['settings'][$vid]['use_global'] = [
          '#type' => 'value',
          '#value' => 0,
        ];
      }
      $element['settings'][$vid]['term_reference'] = [
        '#type' => 'entity_autocomplete',
        '#description' => $this->t("Separate terms using a comma ','."),
        '#target_type' => 'taxonomy_term',
        '#selection_handler' => 'default:taxonomy_term',
        '#selection_settings' => [
          'target_bundles' => [
            $vid => $vid,
          ],
          'sort' => [
            'field' => 'name',
            'direction' => 'ASC',
          ],
          'auto_create' => FALSE,
          'auto_create_bundle' => NULL,
          'match_operator' => 'CONTAINS',
        ],
        '#default_value' => $default_referenced_terms,
        '#tags' => TRUE,
        '#maxlength' => 512,
        '#states' => [
          'invisible' => $states,
        ],
      ];
      if (!empty($default_referenced_terms) || $default_use_global === TRUE) {
        $element['settings']['#open'] = TRUE;
      }
    }

    return $element;
  }

  /**
   * List of vocabularies and all the bundles that don't reference them.
   *
   * @param array $vocabularies
   *   Vocabulary list.
   * @param array $entity_bundle_ids
   *   List of bundles for this entity type.
   * @param string $entity_type
   *   Id of the entity_type.
   *
   * @return array
   *   Return non-related bundles for each vocabulary.
   */
  protected function getVocabularyUnrelatedBundles(array $vocabularies, array $entity_bundle_ids, $entity_type) {
    $vocab_entity_types = [];
    foreach ($vocabularies as $id => $vocab) {
      $vocab_entity_types[$id] = array_combine($entity_bundle_ids, $entity_bundle_ids);
    }
    foreach ($entity_bundle_ids as $entity_bundle) {
      if ($entity_bundle === '_none') {
        continue;
      }
      $fields = $this->entityFieldManager->getFieldDefinitions($entity_type, $entity_bundle);
      foreach ($fields as $field_info) {
        /** @var \Drupal\field\Entity\FieldConfig $fieldItem */
        if ('entity_reference' === $field_info->getType()) {
          $settings = $field_info->getSettings();
          if (isset($settings['target_type']) && 'taxonomy_term' === $settings['target_type']) {
            if (!empty($settings['handler_settings']['target_bundles'])) {
              foreach ($settings['handler_settings']['target_bundles'] as $vocab_name) {
                unset($vocab_entity_types[$vocab_name][$entity_bundle]);
              }
            }
          }
        }
      }
    }
    $vocab_entity_types = array_filter($vocab_entity_types);

    return $vocab_entity_types;
  }

}
